@@ -1,3 +1,4 @@
+from collections.abc import Sequence
 import json
 import os
 import sys
@@ -7,7 +8,6 @@ import yaml
 from pathlib import Path
 from pydoc import pipepager, pager
 import re
-
 import spyctl.spyctl_lib as lib
 
 yaml.Dumper.ignore_aliases = lambda *args: True
@@ -84,6 +84,7 @@ def show(
     alternative_outputs: Dict[str, Callable] = {},
     dest=lib.OUTPUT_DEST_STDOUT,
     output_fn=None,
+    ndjson=False,
 ):
     """Display or save python object
 
@@ -97,6 +98,8 @@ def show(
             lib.OUTPUT_DEST_STDOUT.
         output_fn (str, optional): Filename if outputting to a file.
             Defaults to None.
+        ndjson (bool, optional): If output is json, output the json on a
+            single line. Defaults to False
     """
     out_data = None
     if output == lib.OUTPUT_YAML:
@@ -104,7 +107,16 @@ def show(
         if output_fn:
             output_fn += ".yaml"
     elif output == lib.OUTPUT_JSON:
-        out_data = json.dumps(obj, sort_keys=False, indent=2)
+        if ndjson:
+            if _seq_but_not_str(obj):
+                out_data = []
+                for item in obj:
+                    out_data.append(json.dumps(item))
+                out_data = "\n".join(out_data)
+            else:
+                out_data = json.dumps(obj)
+        else:
+            out_data = json.dumps(obj, sort_keys=False, indent=2)
         if output_fn:
             output_fn += ".json"
     elif output == lib.OUTPUT_RAW:
@@ -183,8 +195,8 @@ def time_input(args):
         return t, t
 
 
-def err_exit(message: str):
-    lib.err_exit(message)
+def err_exit(message: str, exception: Exception = None):
+    lib.err_exit(message, exception)
 
 
 def output_to_pager(text: str):
@@ -214,3 +226,9 @@ ANSI_ESCAPE = re.compile(
 def strip_color(text: str):
     rv = ANSI_ESCAPE.sub("", text)
     return rv
+
+
+def _seq_but_not_str(obj):
+    return isinstance(obj, Sequence) and not isinstance(
+        obj, (str, bytes, bytearray)
+    )
