@@ -1,6 +1,7 @@
 import fnmatch
 import ipaddress as ipaddr
 import re
+import time
 from copy import deepcopy
 from difflib import SequenceMatcher
 from pathlib import Path
@@ -9,8 +10,8 @@ from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 import yaml
 
 import spyctl.cli as cli
+import spyctl.schemas_v2 as schemas
 import spyctl.spyctl_lib as lib
-import spyctl.schemas as schemas
 
 SPEC_FIELD = lib.SPEC_FIELD
 CONT_SELECTOR_FIELD = lib.CONT_SELECTOR_FIELD
@@ -18,6 +19,8 @@ SVC_SELECTOR_FIELD = lib.SVC_SELECTOR_FIELD
 POD_SELECTOR_FIELD = lib.POD_SELECTOR_FIELD
 MACHINE_SELECTOR_FIELD = lib.MACHINE_SELECTOR_FIELD
 NAMESPACE_SELECTOR_FIELD = lib.NAMESPACE_SELECTOR_FIELD
+USER_SELECTOR_FIELD = lib.USER_SELECTOR_FIELD
+TRACE_SELECTOR_FIELD = lib.TRACE_SELECTOR_FIELD
 MATCH_LABELS_FIELD = lib.MATCH_LABELS_FIELD
 PROC_POLICY_FIELD = lib.PROC_POLICY_FIELD
 NET_POLICY_FIELD = lib.NET_POLICY_FIELD
@@ -41,9 +44,6 @@ MERGING_NODE_LIST = None
 
 ADD_START = "+ "
 SUB_START = "- "
-ADD_COLOR = "\x1b[38;5;35m"
-SUB_COLOR = "\x1b[38;5;203m"
-COLOR_END = "\x1b[0m"
 LIST_MARKER = "- "
 DEFAULT_WHITESPACE = "  "
 NET_POL_FIELDS = {lib.INGRESS_FIELD, lib.EGRESS_FIELD}
@@ -65,6 +65,8 @@ class MergeObject:
         merge_schemas: List["MergeSchema"],
         validation_fn: Callable,
         merge_network: bool = True,
+        disable_procs: str = None,
+        disable_conns: str = None,
     ) -> None:
         self.original_obj = deepcopy(obj_data)
         self.obj_data = deepcopy(obj_data)
@@ -73,6 +75,10 @@ class MergeObject:
         self.starting_yaml = yaml.dump(obj_data)
         self.merge_network = merge_network
 
+        # guardian spec merge settings
+        self.__parse_disable_procs_settings(disable_procs)
+        self.__parse_disable_conns_settings(disable_conns)
+
     def symmetric_merge(self, other: Union["MergeObject", Dict]):
         global BASE_NODE_LIST, MERGING_NODE_LIST
         BASE_NODE_LIST = None
@@ -119,15 +125,65 @@ class MergeObject:
     def is_valid(self) -> bool:
         return schemas.valid_object(self.obj_data)
 
-    def get_diff(self) -> Optional[str]:
+    def get_diff(self, full_diff=False) -> Optional[str]:
         original_yaml: str = yaml.dump(self.original_obj, sort_keys=False)
         yaml_lines = original_yaml.splitlines()
         diff_all_fields(self.original_obj, self.obj_data, yaml_lines)
-        return "\n".join(yaml_lines)
+        if full_diff:
+            return "\n".join(yaml_lines)
+        else:
+            summary = []
+            in_sum = set()
+            found_spec = False
+
+            def add_line_to_sum(index, sum_line):
+                if index not in in_sum:
+                    if (
+                        index != 0
+                        and index - 1 >= 0
+                        and index - 1 not in in_sum
+                    ):
+                        summary.append("...")
+                    summary.append(sum_line)
+                    in_sum.add(index)
+
+            for i, line in enumerate(yaml_lines):
+                stripped = line.strip()
+                if (
+                    line.startswith(f"{lib.ADD_COLOR}{ADD_START}")
+                    or line.startswith(f"{ADD_START}")
+                    or line.startswith(f"{lib.SUB_COLOR}{SUB_START}")
+                    or line.startswith(f"{SUB_START}")
+                    or not found_spec
+                    or stripped.startswith(lib.PROC_POLICY_FIELD)
+                    or stripped.startswith(lib.NET_POLICY_FIELD)
+                    or stripped.startswith(lib.INGRESS_FIELD)
+                    or stripped.startswith(lib.EGRESS_FIELD)
+                ):
+                    if line.strip().startswith(lib.SPEC_FIELD):
+                        found_spec = True
+                    for x in range(i - 5, i):
+                        if x >= 0:
+                            add_line_to_sum(x, yaml_lines[x])
+                    add_line_to_sum(i, line)
+                    for x in range(i + 1, i + 6):
+                        if x < len(yaml_lines):
+                            add_line_to_sum(x, yaml_lines[x])
+            if max(in_sum) != len(yaml_lines) - 1:
+                summary.append("...")
+            return "\n".join(summary)
 
     def get_obj_data(self):
         return self.obj_data
 
+    def update_latest_timestamp(self, timestamp=None):
+        meta = self.obj_data.get(lib.METADATA_FIELD)
+        if meta and lib.LATEST_TIMESTAMP_FIELD in meta:
+            if timestamp:
+                meta[lib.LATEST_TIMESTAMP_FIELD] = timestamp
+            else:
+                meta[lib.LATEST_TIMESTAMP_FIELD] = time.time()
+
     def __merge_subfields(
         self,
         data: Optional[Dict],
@@ -209,7 +265,7 @@ class MergeObject:
             if data is None or other_data is None:
                 result = None
             else:
-                result = func(data, other_data, symmetric)
+                result = func(self, data, other_data, symmetric)
         else:
             if data is None and other_data is None:
                 result = None
@@ -218,9 +274,55 @@ class MergeObject:
             if other_data is None:
                 result = data
             else:
-                result = func(data, other_data, symmetric)
+                result = func(self, data, other_data, symmetric)
         return result
 
+    def __parse_disable_procs_settings(self, s: str) -> bool:
+        dpf = self.original_obj.get(lib.SPEC_FIELD, {}).get(
+            lib.DISABLE_PROCS_FIELD, ""
+        )
+        if s == lib.DISABLE_PROCS_ALL or dpf == lib.DISABLE_PROCS_ALL:
+            self.disable_procs = lib.DISABLE_PROCS_ALL
+        else:
+            self.disable_procs = None
+
+    def __parse_disable_conns_settings(self, s: str) -> bool:
+        dcf = self.original_obj.get(lib.SPEC_FIELD, {}).get(
+            lib.DISABLE_CONNS_FIELD, ""
+        )
+        if s == lib.DISABLE_CONNS_ALL or dcf == lib.DISABLE_CONNS_ALL:
+            self.disable_conns = lib.DISABLE_CONNS_ALL
+        elif s == lib.DISABLE_CONNS_EGRESS or dcf == lib.EGRESS_FIELD:
+            self.disable_conns = lib.EGRESS_FIELD
+        elif s == lib.DISABLE_CONNS_INGRESS or dcf == lib.INGRESS_FIELD:
+            self.disable_conns = lib.INGRESS_FIELD
+        else:
+            self.disable_conns = None
+
+        dpr = self.original_obj.get(lib.SPEC_FIELD, {}).get(
+            lib.DISABLE_PR_CONNS_FIELD, ""
+        )
+        if s == lib.DISABLE_CONNS_PRIVATE or dpr == lib.DISABLE_CONNS_ALL:
+            self.disable_private_conns = lib.DISABLE_CONNS_ALL
+        elif s == lib.DISABLE_CONNS_PRIVATE_E or dpr == lib.EGRESS_FIELD:
+            self.disable_private_conns = lib.EGRESS_FIELD
+        elif s == lib.DISABLE_CONNS_PRIVATE_I or dpr == lib.INGRESS_FIELD:
+            self.disable_private_conns = lib.INGRESS_FIELD
+        else:
+            self.disable_private_conns = None
+
+        dpu = self.original_obj.get(lib.SPEC_FIELD, {}).get(
+            lib.DISABLE_PU_CONNS_FIELD, ""
+        )
+        if s == lib.DISABLE_CONNS_PUBLIC or dpu == lib.DISABLE_CONNS_ALL:
+            self.disable_public_conns = lib.DISABLE_CONNS_ALL
+        elif s == lib.DISABLE_CONNS_PUBLIC_E or dpu == lib.EGRESS_FIELD:
+            self.disable_public_conns = lib.EGRESS_FIELD
+        elif s == lib.DISABLE_CONNS_PUBLIC_I or dpu == lib.INGRESS_FIELD:
+            self.disable_public_conns = lib.INGRESS_FIELD
+        else:
+            self.disable_public_conns = None
+
 
 class MergeSchema:
     def __init__(
@@ -352,9 +454,14 @@ class ProcessNode:
 
     def __eq__(self, other):
         if isinstance(other, __class__):
-            if not fnmatch.fnmatch(other.name, self.name):
+            if not (
+                fnmatch.fnmatch(other.name, self.name)
+                or fnmatch.fnmatch(self.name, other.name)
+            ):
                 return False
-            if not self.__match_exes(other.exes, strict=True):
+            if not self.__match_exes(
+                other.exes, strict=True, single_match=True
+            ):
                 return False
             if not self.__match_eusers(other.eusers):
                 return False
@@ -374,6 +481,7 @@ class ProcessNode:
                 exe_name = Path(exe).name
                 if fnmatch.fnmatch(other_name, exe_name):
                     return True
+        return False
 
     def __match_exes(
         self, other_exes: str, strict=False, single_match=False
@@ -448,7 +556,7 @@ class ProcessNode:
 
 class ProcessNodeList:
     def __init__(self, nodes_data: List[Dict]) -> None:
-        self.proc_nodes: Dict[str, ProcessNode] = {}
+        self.proc_nodes: Dict[str, ProcessNode] = {}  # id > ProcessNode
         self.proc_name_index: Dict[str, List[str]] = {}
         self.roots: List[ProcessNode] = []
         self.ids = set()
@@ -557,6 +665,25 @@ class ProcessNodeList:
     def __add_node(
         self, node_data: Dict, eusers=[], parent=None
     ) -> "ProcessNode":
+        if "policyNode" in node_data:
+            id = node_data["policyNode"]["id"]
+            children = node_data["policyNode"].get(lib.CHILDREN_FIELD)
+            if id not in BASE_NODE_LIST.ids:
+                raise InvalidMergeError(
+                    f"Deviation process node ID ({id}) is missing in base"
+                    " policy"
+                )
+            node = BASE_NODE_LIST.get_node(id)
+            node_data = node.node.copy()
+            node_data.pop(lib.CHILDREN_FIELD, None)
+            if children:
+                node_data[lib.CHILDREN_FIELD] = children
+            while node.parent:
+                parent = BASE_NODE_LIST.get_node(node.parent)
+                parent_data = parent.node.copy()
+                parent_data[lib.CHILDREN_FIELD] = [node_data]
+                node_data = parent_data
+                node = parent
         proc_node = ProcessNode(self, node_data, eusers, parent)
         self.proc_nodes[proc_node.id] = proc_node
         self.proc_name_index.setdefault(proc_node.name, [])
@@ -616,7 +743,11 @@ class InvalidNetworkNode(Exception):
 
 
 class IPBlock:
-    def __init__(self, ip_network, except_networks: List = None) -> None:
+    def __init__(
+        self,
+        ip_network: Union[ipaddr.IPv4Network, ipaddr.IPv6Network],
+        except_networks: List = None,
+    ) -> None:
         self.network = ip_network
         self.except_networks = except_networks
         if except_networks is not None:
@@ -716,15 +847,54 @@ class NetworkNode:
             )
         self.__parse_port_block(node_data[lib.PORTS_FIELD])
 
-    def symmetrical_merge(self, other_node: "NetworkNode"):
-        self.__merge_ip_blocks(other_node.ip_blocks, symmetrical=True)
-        self.__merge_dns_names(other_node.dns_names, symmetrical=True)
+    def symmetrical_merge(
+        self,
+        other_node: "NetworkNode",
+    ):
+        self.__merge_ip_blocks(
+            other_node.ip_blocks,
+            symmetrical=True,
+        )
+        self.__merge_dns_names(
+            other_node.dns_names,
+            symmetrical=True,
+        )
+        self.__merge_process_ids(other_node.processes)
 
     def asymmetrical_merge(self, other_node: "NetworkNode"):
-        self.__merge_ip_blocks(other_node.ip_blocks, symmetrical=False)
-        self.__merge_dns_names(other_node.dns_names, symmetrical=False)
+        self.__merge_ip_blocks(
+            other_node.ip_blocks,
+            symmetrical=False,
+        )
+        self.__merge_dns_names(
+            other_node.dns_names,
+            symmetrical=False,
+        )
+        self.__merge_process_ids(other_node.processes)
 
-    def as_dict(self) -> Dict:
+    def internal_merge(self):
+        if self.node_list.ignore_procs:
+            self.proc_node_list = []
+        new_blocks = []
+        for block in self.ip_blocks:
+            if block in new_blocks:
+                continue
+            if self.node_list.ignore_private and block.network.is_private:
+                continue
+            if self.node_list.ignore_public and not block.network.is_private:
+                continue
+            new_blocks.append(block)
+        self.ip_blocks = new_blocks
+        dns_names = set()
+        for dns_name in self.dns_names:
+            if self.node_list.ignore_private and lib.is_private_dns(dns_name):
+                continue
+            if self.node_list.ignore_public and lib.is_public_dns(dns_name):
+                continue
+            dns_names.add(dns_name)
+        self.dns_names = list(dns_names)
+
+    def as_dict(self) -> Optional[Dict]:
         or_field_string = (
             lib.TO_FIELD if self.type == NODE_TYPE_EGRESS else lib.FROM_FIELD
         )
@@ -746,8 +916,12 @@ class NetworkNode:
         ]
         ipv6_blocks.sort(key=lambda x: x.network)
         ipv6_blocks = [b.as_dict() for b in ipv6_blocks]
-        rv[or_field_string] = dns_names + ipv4_blocks + ipv6_blocks
-        rv[lib.PROCESSES_FIELD] = self.processes
+        or_blocks = dns_names + ipv4_blocks + ipv6_blocks
+        if not or_blocks:
+            return None
+        rv[or_field_string] = or_blocks
+        if self.processes:
+            rv[lib.PROCESSES_FIELD] = self.processes
         rv[lib.PORTS_FIELD] = [p.as_dict() for p in self.port_ranges]
         return rv
 
@@ -761,15 +935,18 @@ class NetworkNode:
             NetworkNode: a converted copy of this network node
         """
         rv = deepcopy(self)
-        for i, id in enumerate(rv.processes.copy()):
-            proc_node = rv.__find_proc_node(id, self.proc_node_list)
-            if proc_node is None:
-                raise InvalidNetworkNode(
-                    "Unable to find process node to convert"
-                )
-            if proc_node.merged_id is not None:
-                rv.processes[i] = proc_node.merged_id
-        rv.processes = list(set(rv.processes))
+        if self.node_list.ignore_procs:
+            rv.processes = []
+        else:
+            for i, id in enumerate(rv.processes.copy()):
+                proc_node = rv.__find_proc_node(id, self.proc_node_list)
+                if proc_node is None:
+                    raise InvalidNetworkNode(
+                        "Unable to find process node to convert"
+                    )
+                if proc_node.merged_id is not None:
+                    rv.processes[i] = proc_node.merged_id
+            rv.processes = list(set(rv.processes))
         rv.proc_node_list = BASE_NODE_LIST
         return rv
 
@@ -823,9 +1000,18 @@ class NetworkNode:
                 )
             )
 
-    def __merge_ip_block(self, other_ip_block: IPBlock, symmetrical=False):
-        # if other_ip_block in self.ip_blocks:
-        #     return
+    def __merge_process_ids(self, other_processes: List[str]):
+        if self.node_list.ignore_procs:
+            self.processes = []
+            return
+        self.processes.extend(other_processes)
+        self.processes = sorted(list(set(self.processes)))
+
+    def __merge_ip_block(
+        self,
+        other_ip_block: IPBlock,
+        symmetrical=False,
+    ):
         if symmetrical:
             match = False
             for i, ip_block in enumerate(self.ip_blocks):
@@ -848,12 +1034,25 @@ class NetworkNode:
                 self.ip_blocks.append(other_ip_block)
 
     def __merge_ip_blocks(
-        self, other_ip_blocks: List[IPBlock], symmetrical=False
+        self,
+        other_ip_blocks: List[IPBlock],
+        symmetrical=False,
     ):
         for o_ip_block in other_ip_blocks:
+            if self.node_list.ignore_private and o_ip_block.network.is_private:
+                continue
+            if (
+                self.node_list.ignore_public
+                and not o_ip_block.network.is_private
+            ):
+                continue
             self.__merge_ip_block(o_ip_block, symmetrical)
 
-    def __merge_dns_name(self, other_dns_name: str, symmetrical=False):
+    def __merge_dns_name(
+        self,
+        other_dns_name: str,
+        symmetrical=False,
+    ):
         if symmetrical:
             match = False
             for i, dns_name in enumerate(self.dns_names):
@@ -875,9 +1074,19 @@ class NetworkNode:
             if not match:
                 self.dns_names.append(other_dns_name)
 
-    def __merge_dns_names(self, other_dns_names: List[str], symmetrical=False):
+    def __merge_dns_names(
+        self,
+        other_dns_names: List[str],
+        symmetrical=False,
+    ):
         for o_dns_name in other_dns_names:
-            self.__merge_dns_name(o_dns_name)
+            if self.node_list.ignore_private and lib.is_private_dns(
+                o_dns_name
+            ):
+                continue
+            if self.node_list.ignore_public and lib.is_public_dns(o_dns_name):
+                continue
+            self.__merge_dns_name(o_dns_name, symmetrical)
 
     def __find_proc_node(
         self, proc_id, node_list: ProcessNodeList
@@ -892,6 +1101,13 @@ class NetworkNode:
 
     def __contains_ip_blocks(self, other_ip_blocks: List[IPBlock]) -> bool:
         for o_ip_block in other_ip_blocks:
+            if self.node_list.ignore_private and o_ip_block.network.is_private:
+                continue
+            if (
+                self.node_list.ignore_public
+                and not o_ip_block.network.is_private
+            ):
+                continue
             if not self.__contains_ip_block(o_ip_block):
                 return False
         return True
@@ -918,6 +1134,12 @@ class NetworkNode:
 
     def __contains_dns_names(self, other_dns_names: List[str]) -> bool:
         for o_dns_name in other_dns_names:
+            if self.node_list.ignore_private and lib.is_private_dns(
+                o_dns_name
+            ):
+                continue
+            if self.node_list.ignore_public and lib.is_public_dns(o_dns_name):
+                continue
             if not self.__contains_dns_name(o_dns_name):
                 return False
         return True
@@ -928,16 +1150,27 @@ class NetworkNode:
         other_node = self.__find_proc_node(other_process_id, other_proc_list)
         if other_node is None:
             raise InvalidNetworkNode("Unable to find process node")
+        # First check if the IDs are a match
         cmp_id = other_node.id
         if other_node.merged_id is not None:
             cmp_id = other_node.merged_id
         if cmp_id in self.processes:
             return True
+        # If not a match check that the processes are comparable
+        # Here we disregard place in the process tree and we're
+        # just looking for processes with the same name and exes
+        for node_id in self.processes:
+            node = self.proc_node_list.get_node(node_id)
+            if other_node in node or node in other_node:
+                return True
         return False
 
     def __contains_processes(
         self, other_processes: List[str], other_proc_list: ProcessNodeList
     ) -> bool:
+        if len(self.processes) == 0 or self.node_list.ignore_procs:
+            # processes of len 0 means any process
+            return True
         for o_process_id in other_processes:
             if not self.__contains_process(o_process_id, other_proc_list):
                 return False
@@ -984,25 +1217,69 @@ class NetworkNode:
 
 class NetworkNodeList:
     def __init__(
-        self, nodes_data: List[Dict], proc_node_list: ProcessNodeList
+        self,
+        nodes_data: List[Dict],
+        proc_node_list: ProcessNodeList,
+        ignore_private=False,
+        ignore_public=False,
+        ignore_procs=False,
     ) -> None:
         self.nodes: List[NetworkNode] = []
         self.proc_node_list = proc_node_list
+        self.ignore_private = ignore_private
+        self.ignore_public = ignore_public
+        self.ignore_procs = ignore_procs
         for node_data in nodes_data:
             self.__add_node(node_data)
+        if self.nodes:
+            self.type = self.nodes[0].type
+        else:
+            self.type = None
 
-    def symmetrical_merge(self, other_list: "NetworkNodeList"):
+    def symmetrical_merge(
+        self,
+        other_list: "NetworkNodeList",
+    ):
         for other_node in other_list.nodes:
             self.__symmetrical_merge_helper(other_node)
 
-    def asymmetrical_merge(self, other_list: "NetworkNodeList"):
+    def asymmetrical_merge(
+        self,
+        other_list: "NetworkNodeList",
+    ):
         for other_node in other_list.nodes:
             self.__asymmetrical_merge_helper(other_node)
 
+    def internal_merge(
+        self,
+    ):
+        if len(self.nodes) <= 1:
+            if len(self.nodes) == 1:
+                self.nodes[0].internal_merge()
+            return
+        skip_index = set()
+        new_nodes = []
+        for i, node in enumerate(self.nodes):
+            node.internal_merge()
+            if i in skip_index:
+                continue
+            if i == len(self.nodes) - 1:
+                new_nodes.append(self.nodes[i])
+                break
+            x = i + 1
+            new_nodes.append(node)
+            for j, other_node in enumerate(self.nodes[x:]):
+                if other_node in node or node in other_node:
+                    node.symmetrical_merge(other_node)
+                    skip_index.add(j + x)
+        self.nodes = new_nodes
+
     def get_data(self) -> List[Dict]:
         rv = []
         for node in self.nodes:
-            rv.append(node.as_dict())
+            node_dict = node.as_dict()
+            if node_dict:
+                rv.append(node_dict)
         return rv
 
     def __add_node(self, node_data: Dict):
@@ -1023,7 +1300,10 @@ class NetworkNodeList:
         if not match:
             self.nodes.append(cvt_other_node)
 
-    def __asymmetrical_merge_helper(self, other_node: "NetworkNode"):
+    def __asymmetrical_merge_helper(
+        self,
+        other_node: "NetworkNode",
+    ):
         match = False
         cvt_other_node = other_node.converted
         for i, node in enumerate(self.nodes):
@@ -1036,9 +1316,16 @@ class NetworkNodeList:
 
 
 def merge_proc_policies(
-    proc_data: List[Dict], other_proc_data: List[Dict], symmetric: bool
+    mo: MergeObject,
+    proc_data: List[Dict],
+    other_proc_data: List[Dict],
+    symmetric: bool,
 ):
     global BASE_NODE_LIST, MERGING_NODE_LIST
+    if mo.disable_procs:
+        BASE_NODE_LIST = ProcessNodeList([])
+        MERGING_NODE_LIST = ProcessNodeList([])
+        return []
     if BASE_NODE_LIST is None:
         BASE_NODE_LIST = ProcessNodeList(proc_data)
     MERGING_NODE_LIST = ProcessNodeList(other_proc_data)
@@ -1052,11 +1339,53 @@ def merge_proc_policies(
 
 
 def merge_ingress_or_egress(
-    base_data: List[Dict], other_data: List[Dict], symmetric: bool
+    mo: MergeObject,
+    base_data: List[Dict],
+    other_data: List[Dict],
+    symmetric: bool,
 ):
-    result = []
-    net_node_list = NetworkNodeList(base_data, BASE_NODE_LIST)
-    other_node_list = NetworkNodeList(other_data, MERGING_NODE_LIST)
+    if mo.disable_conns == lib.DISABLE_CONNS_ALL:
+        return []
+    disable_private = mo.disable_private_conns == lib.DISABLE_CONNS_ALL
+    disable_public = mo.disable_public_conns == lib.DISABLE_CONNS_ALL
+    disable_procs = mo.disable_procs == lib.DISABLE_PROCS_ALL
+    net_node_list = NetworkNodeList(
+        base_data,
+        BASE_NODE_LIST,
+        disable_private,
+        disable_public,
+        disable_procs,
+    )
+    other_node_list = NetworkNodeList(
+        other_data,
+        MERGING_NODE_LIST,
+        disable_private,
+        disable_public,
+        disable_procs,
+    )
+    direction = net_node_list.type or other_node_list.type
+    if not direction:
+        return []
+    if direction == lib.EGRESS_FIELD:
+        if mo.disable_conns == lib.DISABLE_CONNS_EGRESS:
+            return []
+        if mo.disable_private_conns == lib.DISABLE_CONNS_EGRESS:
+            net_node_list.ignore_private = True
+            other_node_list.ignore_private = True
+        if mo.disable_public_conns == lib.DISABLE_CONNS_EGRESS:
+            net_node_list.ignore_public = True
+            other_node_list.ignore_public = True
+    else:
+        if mo.disable_conns == lib.DISABLE_CONNS_INGRESS:
+            return []
+        if mo.disable_private_conns == lib.DISABLE_CONNS_INGRESS:
+            net_node_list.ignore_private = True
+            other_node_list.ignore_private = True
+        if mo.disable_public_conns == lib.DISABLE_CONNS_INGRESS:
+            net_node_list.ignore_public = True
+            other_node_list.ignore_public = True
+    net_node_list.internal_merge()
+    other_node_list.internal_merge()
     if symmetric:
         net_node_list.symmetrical_merge(other_node_list)
     else:
@@ -1065,7 +1394,9 @@ def merge_ingress_or_egress(
     return result
 
 
-def common_keys_merge(base_data: Dict, other_data: Dict, symmetric: bool):
+def common_keys_merge(
+    mo: MergeObject, base_data: Dict, other_data: Dict, symmetric: bool
+):
     result = {}
     common_keys = set(base_data).intersection(set(other_data))
     for key, value in base_data.items():
@@ -1078,7 +1409,9 @@ def common_keys_merge(base_data: Dict, other_data: Dict, symmetric: bool):
     return None
 
 
-def wildcard_merge(base_str: str, other_str: str, symmetric: bool):
+def wildcard_merge(
+    mo: MergeObject, base_str: str, other_str: str, symmetric: bool
+):
     "Result of the merge can be wildcarded"
     if symmetric:
         if base_str and other_str:
@@ -1101,7 +1434,7 @@ def wildcard_merge(base_str: str, other_str: str, symmetric: bool):
     return result
 
 
-def all_eq_merge(base_str: str, other_str: str, _):
+def all_eq_merge(mo: MergeObject, base_str: str, other_str: str, _):
     if base_str == other_str:
         result = base_str
     else:
@@ -1109,11 +1442,13 @@ def all_eq_merge(base_str: str, other_str: str, _):
     return result
 
 
-def keep_base_value_merge(base_val: Any, other_val: Any, _):
+def keep_base_value_merge(mo: MergeObject, base_val: Any, other_val: Any, _):
     return base_val
 
 
-def greatest_value_merge(base_val, other_val, symmetric: bool):
+def greatest_value_merge(
+    mo: MergeObject, base_val, other_val, symmetric: bool
+):
     if base_val is not None and other_val is None:
         result = base_val
     elif base_val is None and other_val is not None:
@@ -1127,6 +1462,24 @@ def greatest_value_merge(base_val, other_val, symmetric: bool):
     return result
 
 
+def string_list_merge(
+    mo: MergeObject, base_value: List[str], other_value: List[str], _
+):
+    string_set = set(base_value).union(set(other_value))
+    return sorted(string_set)
+
+
+def unique_dict_list_merge(
+    mo: MergeObject, base_value: List[Dict], other_value: List[dict], _
+):
+    rv = base_value.copy()
+    for item in other_value:
+        if item in base_value:
+            continue
+        rv.append(item)
+    return rv
+
+
 NET_POLICY_MERGE_SCHEMA = MergeSchema(
     NET_POLICY_FIELD,
     merge_functions={
@@ -1175,6 +1528,32 @@ NAMESPACE_SELECTOR_MERGE_SCHEMA = MergeSchema(
     values_required=True,
     is_selector=True,
 )
+TRACE_SELECTOR_MERGE_SCHEMA = MergeSchema(
+    TRACE_SELECTOR_FIELD,
+    merge_functions={
+        lib.TRIGGER_ANCESTORS_FIELD: all_eq_merge,
+        lib.TRIGGER_CLASS_FIELD: all_eq_merge,
+    },
+    values_required=True,
+    is_selector=True,
+)
+USER_SELECTOR_MERGE_SCHEMA = MergeSchema(
+    USER_SELECTOR_FIELD,
+    merge_functions={
+        lib.USERS_FIELD: string_list_merge,
+        lib.INTERACTIVE_USERS_FIELD: string_list_merge,
+        lib.NON_INTERACTIVE_USERS_FIELD: string_list_merge,
+    },
+)
+PROCESS_SELECTOR_MERGE_SCHEMA = MergeSchema(
+    USER_SELECTOR_FIELD,
+    merge_functions={
+        lib.NAME_FIELD: string_list_merge,
+        lib.EXE_FIELD: string_list_merge,
+        lib.EUSER_FIELD: string_list_merge,
+        lib.INTERACTIVE_FIELD: keep_base_value_merge,
+    },
+)
 SPEC_MERGE_SCHEMA = MergeSchema(
     SPEC_FIELD,
     sub_schemas={
@@ -1189,9 +1568,31 @@ SPEC_MERGE_SCHEMA = MergeSchema(
         lib.ENABLED_FIELD: keep_base_value_merge,
         PROC_POLICY_FIELD: merge_proc_policies,
         RESPONSE_FIELD: keep_base_value_merge,
+        lib.DISABLE_PROCS_FIELD: keep_base_value_merge,
+        lib.DISABLE_CONNS_FIELD: keep_base_value_merge,
+        lib.DISABLE_PU_CONNS_FIELD: keep_base_value_merge,
+        lib.DISABLE_PR_CONNS_FIELD: keep_base_value_merge,
+        lib.POL_MODE_FIELD: keep_base_value_merge,
     },
     values_required=True,
 )
+TRACE_SUPPRESSION_SPEC_MERGE_SCHEMA = MergeSchema(
+    SPEC_FIELD,
+    sub_schemas={
+        SVC_SELECTOR_FIELD: SVC_SELECTOR_MERGE_SCHEMA,
+        CONT_SELECTOR_FIELD: CONTAINER_SELECTOR_MERGE_SCHEMA,
+        MACHINE_SELECTOR_FIELD: MACHINE_SELECTOR_MERGE_SCHEMA,
+        POD_SELECTOR_FIELD: POD_SELECTOR_MERGE_SCHEMA,
+        NAMESPACE_SELECTOR_FIELD: NAMESPACE_SELECTOR_MERGE_SCHEMA,
+        TRACE_SELECTOR_FIELD: TRACE_SELECTOR_MERGE_SCHEMA,
+        USER_SELECTOR_FIELD: USER_SELECTOR_MERGE_SCHEMA,
+    },
+    merge_functions={
+        lib.ENABLED_FIELD: keep_base_value_merge,
+        lib.ALLOWED_FLAGS_FIELD: unique_dict_list_merge,
+        RESPONSE_FIELD: keep_base_value_merge,
+    },
+)
 
 
 def make_wildcard(strs: List[str]):
@@ -1203,7 +1604,13 @@ def make_wildcard(strs: List[str]):
     # Simple string match didn't work so lets see if there is a
     # better match (takes more computation)
     original_str = sub_str = strs[0]
+    first_char = original_str[0]
+    last_char = original_str[-1]
     for name in strs[1:]:
+        if first_char != name[0]:
+            first_char = None
+        if last_char != name[-1]:
+            last_char = None
         name = name.strip("*")
         match = SequenceMatcher(None, sub_str, name).find_longest_match(
             0, len(sub_str), 0, len(name)
@@ -1215,12 +1622,18 @@ def make_wildcard(strs: List[str]):
             break
     if len(sub_str) < 3:
         ret = None
-    elif original_str.startswith(sub_str):
-        ret = sub_str + "*"
-    elif original_str.endswith(sub_str):
-        ret = "*" + sub_str
-    else:
+    elif not (match.b == 0 and original_str.startswith(sub_str)) and not (
+        match.b + match.size == len(name) and original_str.endswith(sub_str)
+    ):
         ret = "*" + sub_str + "*"
+    elif not (match.b == 0 and original_str.startswith(sub_str)):
+        ret = "*" + sub_str
+    elif not (
+        match.b + match.size == len(name) and original_str.endswith(sub_str)
+    ):
+        ret = sub_str + "*"
+    else:
+        cli.err_exit(f"Bug detected in wildcard logic. Input:" f" '{strs}'.")
     return ret
 
 
@@ -1229,11 +1642,11 @@ def make_orig_line(line: str) -> str:
 
 
 def make_sub_line(line: str) -> str:
-    return f"{SUB_COLOR}{SUB_START}{line}{COLOR_END}"
+    return f"{lib.SUB_COLOR}{SUB_START}{line}{lib.COLOR_END}"
 
 
 def make_add_line(line: str) -> str:
-    return f"{ADD_COLOR}{ADD_START}{line}{COLOR_END}"
+    return f"{lib.ADD_COLOR}{ADD_START}{line}{lib.COLOR_END}"
 
 
 class DiffLines:
