@@ -10,6 +10,7 @@ import unicodedata
 from base64 import urlsafe_b64encode as b64url
 from datetime import timezone
 from fnmatch import fnmatch
+from hashlib import md5
 from pathlib import Path
 from typing import IO, Any, Dict, Iterable, List, Optional, Tuple, Union
 from uuid import uuid4
@@ -35,14 +36,51 @@ class Aliases:
         return self.name_plural or self.name
 
 
+COLORIZE_OUTPUT = True
 APP_NAME = "spyctl"
 WARNING_MSG = "is_warning"
 WARNING_COLOR = "\x1b[38;5;203m"
+NOTICE_COLOR = "\x1b[38;5;75m"
+ADD_COLOR = "\x1b[38;5;35m"
+SUB_COLOR = "\x1b[38;5;203m"
 COLOR_END = "\x1b[0m"
+API_CALL = False
+DEBUG = False
+LOG_VAR = []
+ERR_VAR = []
+USE_LOG_VARS = False
+
+
+def disable_colorization():
+    global COLORIZE_OUTPUT, WARNING_COLOR, COLOR_END
+    global ADD_COLOR, SUB_COLOR, NOTICE_COLOR
+    COLORIZE_OUTPUT = False
+    WARNING_COLOR = ""
+    COLOR_END = ""
+    SUB_COLOR = ""
+    ADD_COLOR = ""
+    NOTICE_COLOR = ""
+
+
+def flush_log_var() -> str:
+    global LOG_VAR
+    rv = "\n".join(LOG_VAR)
+    LOG_VAR.clear()
+    return rv
+
+
+def flush_err_var() -> str:
+    global ERR_VAR
+    rv = "\n".join(ERR_VAR)
+    ERR_VAR.clear()
+    return rv
+
 
 # Resource Aliases
-CLUSTERS_RESOURCE = Aliases(
-    ["clusters", "cluster", "clust", "clusts", "clus"], "cluster", "clusters"
+AGENT_RESOURCE = Aliases(
+    ["agents", "agent", "ag"],
+    "agent",
+    "agents",
 )
 BASELINES_RESOURCE = Aliases(
     [
@@ -57,68 +95,13 @@ BASELINES_RESOURCE = Aliases(
     "baseline",
     "baselines",
 )
-DEPLOYMENTS_RESOURCE = Aliases(
-    ["deployments", "deployment", "deploys", "deploy"],
-    "deployment",
-    "deployments",
+CLUSTERS_RESOURCE = Aliases(
+    ["clusters", "cluster", "clust", "clusts", "clus"], "cluster", "clusters"
 )
-NAMESPACES_RESOURCE = Aliases(
-    ["namespaces", "name", "names", "namesp", "namesps", "namespace"],
-    "namespace",
-    "namespaces",
-)
-MACHINES_RESOURCE = Aliases(
-    ["machines", "mach", "machs", "machine"],
-    "machine",
-    "machines",
-)
-NODES_RESOURCE = Aliases(["nodes", "node"], "node", "nodes")
-PODS_RESOURCE = Aliases(["pods", "pod"], "pod", "pods")
-REDFLAGS_RESOURCE = Aliases(["redflags", "redflag"], "redflag", "redflags")
-OPSFLAGS_RESOURCE = Aliases(["opsflags", "opsflag"], "opsflag", "opsflags")
-FINGERPRINTS_RESOURCE = Aliases(
-    [
-        "fingerprints",
-        "print",
-        "prints",
-        "fingerprint",
-        "fprint",
-        "f",
-        "fprints",
-        "fingerprint-group",
-        "fingerprint-groups",
-    ],
-    "fingerprint",
-    "fingerprints",
-)
-POLICIES_RESOURCE = Aliases(
-    [
-        "policies",
-        "spyderbat-policy",
-        "spyderbat-policies",
-        "spy-pol",
-        "spol",
-        "sp",
-        "policy",
-        "pol",
-        "p",
-    ],
-    "policy",
-    "policies",
-)
-PROCESSES_RESOURCE = Aliases(
-    [
-        "processes",
-        "process",
-        "proc",
-        "procs",
-        "pro",
-        "pros",
-        "pces",
-        "pcess",
-    ],
-    "process",
-    "processes",
+CONTAINER_RESOURCE = Aliases(
+    ["container", "containers", "cont" "c"],
+    "container",
+    "containers",
 )
 CONNECTIONS_RESOURCE = Aliases(
     [
@@ -134,6 +117,111 @@ CONNECTIONS_RESOURCE = Aliases(
     "connection",
     "connections",
 )
+CONNECTION_BUN_RESOURCE = Aliases(
+    ["connection-bundle", "connection-bundles", "conn_bun", "conn_buns", "cb"],
+    "connection-bundle",
+    "connection-bundles",
+)
+DEPLOYMENTS_RESOURCE = Aliases(
+    ["deployments", "deployment", "deploys", "deploy"],
+    "deployment",
+    "deployments",
+)
+DEVIATIONS_RESOURCE = Aliases(
+    ["deviations", "deviation", "dev"],
+    "deviation",
+    "deviations",
+)
+FINGERPRINT_GROUP_RESOURCE = Aliases(
+    ["fingerprint-group", "fingerprint-groups", "fprint-group", "fg"],
+    "fingerprint-group",
+    "fingerprint-groups",
+)
+FINGERPRINTS_RESOURCE = Aliases(
+    [
+        "fingerprints",
+        "print",
+        "prints",
+        "fingerprint",
+        "fprint",
+        "f",
+        "fprints",
+        "fingerprint-group",
+        "fingerprint-groups",
+    ],
+    "fingerprint",
+    "fingerprints",
+)
+MACHINES_RESOURCE = Aliases(
+    ["machines", "mach", "machs", "machine"],
+    "machine",
+    "machines",
+)
+NAMESPACES_RESOURCE = Aliases(
+    ["namespaces", "name", "names", "namesp", "namesps", "namespace"],
+    "namespace",
+    "namespaces",
+)
+NODES_RESOURCE = Aliases(["nodes", "node"], "node", "nodes")
+OPSFLAGS_RESOURCE = Aliases(["opsflags", "opsflag"], "opsflag", "opsflags")
+PODS_RESOURCE = Aliases(["pods", "pod"], "pod", "pods")
+REDFLAGS_RESOURCE = Aliases(["redflags", "redflag"], "redflag", "redflags")
+POLICIES_RESOURCE = Aliases(
+    [
+        "policies",
+        "spyderbat-policy",
+        "spyderbat-policies",
+        "spy-pol",
+        "spol",
+        "policy",
+        "pol",
+        "p",
+    ],
+    "policy",
+    "policies",
+)
+PROCESSES_RESOURCE = Aliases(
+    [
+        "processes",
+        "process",
+        "proc",
+        "procs",
+    ],
+    "process",
+    "processes",
+)
+SOURCES_RESOURCE = Aliases(["source", "sources", "src"], "source", "sources")
+SPYDERTRACE_RESOURCE = Aliases(
+    ["spydertrace", "spydertraces", "spyder", "trace", "traces"],
+    "spydertrace",
+    "spydertraces",
+)
+SPYDERTRACE_SUMMARY_RESOURCE = Aliases(
+    [
+        "spydertrace-summary",
+        "spydertrace-summaries",
+        "trace-summary",
+        "trace-summaries",
+        "t-sum",
+    ],
+    "spydertrace-summary",
+    "spydertrace-summaries",
+)
+SUPPRESSION_POLICY_RESOURCE = Aliases(
+    [
+        "suppression-policy",
+        "suppression-policies",
+        "sp",
+        "s-pol",
+        "trace-policy",
+    ],
+    "suppression-policy",
+    "suppression-policies",
+)
+UID_LIST_RESOURCE = Aliases(
+    ["uid-list", "uid-lists", "uid", "uids-list"], "uid-list", "uid-lists"
+)
+
 SECRETS_ALIAS = Aliases(["secret", "secrets", "sec", "s"], "secret", "secrets")
 CONFIG_ALIAS = Aliases(
     ["config", "configs", "conf", "cfg", "configuration", "configurations"],
@@ -141,11 +229,35 @@ CONFIG_ALIAS = Aliases(
     "configs",
 )
 
-DEL_RESOURCES: List[str] = [POLICIES_RESOURCE.name]
+ALL_RESOURCES: List[Aliases] = [
+    g_var
+    for g_var_name, g_var in globals().items()
+    if g_var_name.endswith("RESOURCE")
+]
+
+
+def get_plural_name_from_alias(alias: str):
+    for resource in ALL_RESOURCES:
+        if alias == resource:
+            return resource.name_plural
+    return None
+
+
+DEL_RESOURCES: List[str] = [
+    POLICIES_RESOURCE.name,
+    SUPPRESSION_POLICY_RESOURCE.name,
+]
+DESC_RESOURCES: List[str] = [
+    POLICIES_RESOURCE.name,
+]
 GET_RESOURCES: List[str] = [
+    AGENT_RESOURCE.name_plural,
     CLUSTERS_RESOURCE.name_plural,
     CONNECTIONS_RESOURCE.name_plural,
+    CONNECTION_BUN_RESOURCE.name_plural,
+    CONTAINER_RESOURCE.name_plural,
     DEPLOYMENTS_RESOURCE.name_plural,
+    DEVIATIONS_RESOURCE.name_plural,
     FINGERPRINTS_RESOURCE.name_plural,
     MACHINES_RESOURCE.name_plural,
     NAMESPACES_RESOURCE.name_plural,
@@ -155,13 +267,47 @@ GET_RESOURCES: List[str] = [
     POLICIES_RESOURCE.name_plural,
     PROCESSES_RESOURCE.name_plural,
     REDFLAGS_RESOURCE.name_plural,
+    SOURCES_RESOURCE.name_plural,
+    # SPYDERTRACE_SUMMARY_RESOURCE.name_plural,
+    SUPPRESSION_POLICY_RESOURCE.name_plural,
+    CONTAINER_RESOURCE.name_plural,
+    SPYDERTRACE_RESOURCE.name_plural,
 ]
+LOGS_RESOURCES: List[str] = [POLICIES_RESOURCE.name]
 VAL_RESOURCES: List[str] = [
     BASELINES_RESOURCE.name,
     POLICIES_RESOURCE.name,
     SECRETS_ALIAS.name,
     CONFIG_ALIAS.name,
 ]
+RESOURCES_WITH_SCHEMAS = [
+    BASELINES_RESOURCE.name,
+    CONFIG_ALIAS.name,
+    FINGERPRINTS_RESOURCE.name,
+    FINGERPRINT_GROUP_RESOURCE.name,
+    POLICIES_RESOURCE.name,
+    SECRETS_ALIAS.name,
+    SUPPRESSION_POLICY_RESOURCE.name,
+    UID_LIST_RESOURCE.name,
+]
+
+CMD_ORG_FIELD = "org"
+
+
+def tmp_context_options(function):
+    function = click.option(f"--{CMD_ORG_FIELD}", hidden=True)(function)
+    function = click.option(f"--{API_KEY_FIELD}", hidden=True)(function)
+    function = click.option(f"--{API_URL_FIELD}", hidden=True)(function)
+    return function
+
+
+def colorization_option(function):
+    function = click.option(
+        "--colorize/--no-colorize",
+        help="Specify coloration on or off. Default is on.",
+        default=True,
+    )(function)
+    return function
 
 
 class DelResourcesParam(click.ParamType):
@@ -177,6 +323,19 @@ class DelResourcesParam(click.ParamType):
         ]
 
 
+class DescribeResourcesParam(click.ParamType):
+    name = "del_resources"
+
+    def shell_complete(
+        self, ctx: click.Context, param: click.Parameter, incomplete: str
+    ) -> List["CompletionItem"]:
+        return [
+            CompletionItem(resrc_name)
+            for resrc_name in DESC_RESOURCES
+            if resrc_name.startswith(incomplete)
+        ]
+
+
 class GetResourcesParam(click.ParamType):
     name = "get_resources"
 
@@ -190,6 +349,32 @@ class GetResourcesParam(click.ParamType):
         ]
 
 
+class LogsResourcesParam(click.ParamType):
+    name = "logs_resources"
+
+    def shell_complete(
+        self, ctx: click.Context, param: click.Parameter, incomplete: str
+    ) -> List["CompletionItem"]:
+        return [
+            CompletionItem(resrc_name)
+            for resrc_name in LOGS_RESOURCES
+            if resrc_name.startswith(incomplete)
+        ]
+
+
+class SuppressionPolTypeParam(click.ParamType):
+    name = "get_resources"
+
+    def shell_complete(
+        self, ctx: click.Context, param: click.Parameter, incomplete: str
+    ) -> List["CompletionItem"]:
+        return [
+            CompletionItem(resrc_name)
+            for resrc_name in [POL_TYPE_TRACE]
+            if resrc_name.startswith(incomplete)
+        ]
+
+
 class LabelParam(click.ParamType):
     def convert(
         self,
@@ -219,6 +404,28 @@ class ListParam(click.ParamType):
         return rv
 
 
+class ListDictParam(click.ParamType):
+    def convert(
+        self,
+        value: Any,
+        param: Optional[click.Parameter],
+        ctx: Optional[click.Context],
+    ) -> Any:
+        rv = []
+        rv_dict = {}
+        if "," in value:
+            args_list = value.split(",")
+        for v in args_list:
+            if "=" in v:
+                key, val = v.split("=")[:2]
+                rv_dict[key] = val
+            else:
+                rv.append(v)
+        if rv_dict:
+            rv.append(rv_dict)
+        return rv
+
+
 class FileList(click.File):
     def convert(
         self,
@@ -271,17 +478,56 @@ class FileList(click.File):
         return rv
 
 
-# Spyderbat Schema Prefix'
 SCHEMA_FIELD = "schema"
-EVENT_REDFLAG_PREFIX = "event_redflag"
+
+# Spyderbat Event Schema Prefix'
+EVENT_AGENT_METRICS_PREFIX = "event_agentmetrics"
+EVENT_AUDIT_PREFIX = "event_audit"
 EVENT_OPSFLAG_PREFIX = "event_opsflag"
+EVENT_REDFLAG_PREFIX = "event_redflag"
+
+EVENT_AUDIT_SUBTYPE_MAP = {
+    "deviation": "guardian_deviation",
+    "action": "guardian_action",
+    "redflag": "guardian_redflag",
+    "opsflag": "guardian_opsflag",
+}
+
+# Spyderbat Model Schema Prefix'
+MODEL_AGENT_SCHEMA_PREFIX = "model_agent"
+MODEL_CLUSTER_PREFIX = "model_k8s_cluster"
+MODEL_CONNECTION_PREFIX = "model_connection"
+MODEL_CONN_BUN_PREFIX = "model_bundled_connection"
+MODEL_CONTAINER_PREFIX = "model_container"
+MODEL_DEPLOYMENT_PREFIX = "model_k8s_deployment"
 MODEL_FINGERPRINT_PREFIX = "model_fingerprint"
+MODEL_MACHINE_PREFIX = "model_machine"
+MODEL_NODE_PREFIX = "model_k8s_node"
+MODEL_POD_PREFIX = "model_k8s_pod"
+MODEL_PROCESS_PREFIX = "model_process"
+MODEL_SPYDERTRACE_PREFIX = "model_spydertrace"
+
+MODEL_FINGERPRINT_SUBTYPE_MAP = {
+    "container": "container",
+    "linux-service": "linux_svc",
+}
+
+# Datatypes for searching via API
+DATATYPE_AUDIT = "audit"
+DATATYPE_AGENTS = "agent_status"
+DATATYPE_FINGERPRINTS = "fingerprints"
+DATATYPE_K8S = "k8s"
+DATATYPE_REDFLAGS = "redflags"
+DATATYPE_SPYDERGRAPH = "spydergraph"
 
 # Resource Kinds
 POL_KIND = "SpyderbatPolicy"
+SUP_POL_KIND_ALIAS = "SuppressionPolicy"
 BASELINE_KIND = "SpyderbatBaseline"
 FPRINT_KIND = "SpyderbatFingerprint"
 FPRINT_GROUP_KIND = "FingerprintGroup"
+UID_LIST_KIND = "UidList"
+DEVIATION_KIND = "GuardianDeviation"
 
 # CONFIG Kinds
 CONFIG_KIND = "Config"
@@ -338,6 +584,8 @@ POD_LABELS_FIELD = "pod-labels"
 MACHINES_FIELD = "machines"
 DEFAULT_API_URL = "https://api.spyderbat.com"
 POLICY_UID_FIELD = "policy"
+POLICIES_FIELD = "policies"
+
 
 # Response Actions
 ACTION_KILL_POD = "agentKillPod"
@@ -378,7 +626,10 @@ DNS_SELECTOR_FIELD = "dnsSelector"
 MACHINE_SELECTOR_FIELD = "machineSelector"
 NAMESPACE_SELECTOR_FIELD = "namespaceSelector"
 POD_SELECTOR_FIELD = "podSelector"
+PROCESS_SELECTOR_FIELD = "processSelector"
 SVC_SELECTOR_FIELD = "serviceSelector"
+TRACE_SELECTOR_FIELD = "traceSelector"
+USER_SELECTOR_FIELD = "userSelector"
 MATCH_LABELS_FIELD = "matchLabels"
 # Machine Selector Fields
 HOSTNAME_FIELD = "hostname"
@@ -396,29 +647,151 @@ SELECTOR_FIELDS = {
     MACHINE_SELECTOR_FIELD,
     NAMESPACE_SELECTOR_FIELD,
     POD_SELECTOR_FIELD,
+    TRACE_SELECTOR_FIELD,
+    USER_SELECTOR_FIELD,
 }
 
 # Policies/Fingerprints
+BE_POL_UID_FIELD = (
+    "policy_uid"  # not in the policy objects themselves but in other records
+)
 POL_TYPE_CONT = "container"
 POL_TYPE_SVC = "linux-service"
-POL_TYPES = [POL_TYPE_SVC, POL_TYPE_CONT]
+POL_TYPE_TRACE = "trace"
+SUPPRESSION_POL_TYPES = [POL_TYPE_TRACE]
+GUARDIAN_POL_TYPES = [POL_TYPE_CONT, POL_TYPE_SVC]
+POL_TYPES = [POL_TYPE_SVC, POL_TYPE_CONT, POL_TYPE_TRACE]
+POL_MODE_ENFORCE = "enforce"
+POL_MODE_AUDIT = "audit"
+POL_MODES = [POL_MODE_ENFORCE, POL_MODE_AUDIT]
+POL_MODE_FIELD = "mode"
 ENABLED_FIELD = "enabled"
+DISABLE_PROCS_FIELD = "disableProcesses"
+DISABLE_PROCS_ALL = "all"
+DISABLE_PROCS_STRINGS = [DISABLE_PROCS_ALL]
+DISABLE_CONNS_ALL = "all"
+DISABLE_CONNS_INGRESS = "ingress"
+DISABLE_CONNS_EGRESS = "egress"
+DISABLE_CONNS_PRIVATE = "private"
+DISABLE_CONNS_PRIVATE_E = "private-egress"
+DISABLE_CONNS_PRIVATE_I = "private-ingress"
+DISABLE_CONNS_PUBLIC = "public"
+DISABLE_CONNS_PUBLIC_E = "public-egress"
+DISABLE_CONNS_PUBLIC_I = "public-ingress"
+DISABLE_CONNS_STRINGS = [
+    DISABLE_CONNS_ALL,
+    DISABLE_CONNS_EGRESS,
+    DISABLE_CONNS_INGRESS,
+]
+DISABLE_CONN_OPTIONS_STRINGS = [
+    DISABLE_CONNS_ALL,
+    DISABLE_CONNS_EGRESS,
+    DISABLE_CONNS_INGRESS,
+    DISABLE_CONNS_PRIVATE,
+    DISABLE_CONNS_PRIVATE_E,
+    DISABLE_CONNS_PRIVATE_I,
+    DISABLE_CONNS_PUBLIC,
+    DISABLE_CONNS_PUBLIC_E,
+    DISABLE_CONNS_PUBLIC_I,
+]
+DISABLE_CONNS_FIELD = "disableConnections"
+DISABLE_PR_CONNS_FIELD = "disablePrivateConnections"
+DISABLE_PU_CONNS_FIELD = "disablePublicConnections"
 METADATA_NAME_FIELD = "name"
 METADATA_TAGS_FIELD = "tags"
 METADATA_TYPE_FIELD = "type"
 METADATA_UID_FIELD = "uid"
 METADATA_CREATE_TIME = "creationTimestamp"
 METADATA_NAMESPACE_FIELD = "namespace"
+METADATA_S_CHECKSUM_FIELD = "selectorHash"
+METADATA_START_TIME_FIELD = "startTime"
+METADATA_END_TIME_FIELD = "endTime"
 NET_POLICY_FIELD = "networkPolicy"
 PROC_POLICY_FIELD = "processPolicy"
 FIRST_TIMESTAMP_FIELD = "firstTimestamp"
 LATEST_TIMESTAMP_FIELD = "latestTimestamp"
+TRIGGER_CLASS_FIELD = "triggerClass"
+TRIGGER_ANCESTORS_FIELD = "triggerAncestors"
+USERS_FIELD = "users"
+INTERACTIVE_FIELD = "interactive"
+INTERACTIVE_USERS_FIELD = "interactiveUsers"
+NON_INTERACTIVE_USERS_FIELD = "nonInteractiveUsers"
+ALLOWED_FLAGS_FIELD = "allowedFlags"
+FLAG_SUMMARY_FIELD = "flagSummary"
+FLAGS_FIELD = "flags"
+# For the Spyderbat API
+API_REQ_FIELD_NAME = "name"
+API_REQ_FIELD_POLICY = "policy"
+API_REQ_FIELD_POL_SELECTORS = "selectors"
+API_REQ_FIELD_TAGS = "tags"
+API_REQ_FIELD_TYPE = "type"
+API_REQ_FIELD_UID = "uid"
+API_HAS_TAGS_FIELD = "has_tags"
+# Suppression Policy cmdline fields
+SUP_POL_CMD_TRIG_ANCESTORS = "trigger-ancestors"
+SUP_POL_CMD_TRIG_CLASS = "trigger-class"
+SUP_POL_CMD_USERS = "users"
+SUP_POL_CMD_INT_USERS = "interactive-users"
+SUP_POL_CMD_N_INT_USERS = "non-interactive-users"
+SUP_POL_SELECTOR_FIELDS = [
+    SUP_POL_CMD_TRIG_ANCESTORS,
+    SUP_POL_CMD_TRIG_CLASS,
+    SUP_POL_CMD_USERS,
+    SUP_POL_CMD_INT_USERS,
+    SUP_POL_CMD_N_INT_USERS,
+]
+TRACE_SUMMARY_FIELD = "trace_summary"
+
 NOT_AVAILABLE = "N/A"
 # Fingerprint Groups
 FPRINT_GRP_FINGERPRINTS_FIELD = "fingerprints"
 FPRINT_GRP_CONT_NAMES_FIELD = "containerNames"
 FPRINT_GRP_CONT_IDS_FIELD = "containerIDs"
 FPRINT_GRP_MACHINES_FIELD = "machines"
+# UID List
+UIDS_FIELD = "uniqueIdentifiers"
+
+# Any Object
+VERSION_FIELD = "version"
+VALID_FROM_FIELD = "valid_from"
+
+# K8s Objects
+BE_K8S_STATUS = "k8s_status"
+BE_PHASE = "phase"
+BE_KUID_FIELD = "kuid"
+
+# Connections
+PROC_NAME_FIELD = "proc_name"
+CONN_ID = "id"
+REMOTE_HOSTNAME_FIELD = "remote_hostname"
+PROTOCOL_FIELD = "proto"
+REMOTE_PORT = "remote_port"
+LOCAL_PORT = "local_port"
+
+# Connection Bundles
+CLIENT_IP = "client_ip"
+CLIENT_DNS = "client_dns_name"
+CLIENT_PORT = "client_port"
+SERVER_IP = "server_ip"
+SERVER_DNS = "server_dns_name"
+SERVER_PORT = "server_port"
+NUM_CONNECTIONS = "num_connections"
+
+# Deployments
+REPLICAS_FIELD = "replicas"
+AVAILABLE_REPLICAS_FIELD = "availableReplicas"
+READY_REPLICAS_FIELD = "readyReplicas"
+UPDATED_REPLICAS_FIELD = "updatedReplicas"
+
+# Deviations
+CHECKSUM_FIELD = "checksum"
+
+# Nodes
+NODE_INFO_FIELD = "nodeInfo"
+KUBELET_VERSION_FIELD = "kubeletVersion"
+
+# Pods
+CONTAINER_STATUSES_FIELD = "containerStatuses"
 
 # Processes
 NAME_FIELD = "name"
@@ -428,6 +801,57 @@ EUSER_FIELD = "euser"
 CHILDREN_FIELD = "children"
 LISTENING_SOCKETS = "listeningSockets"
 
+# Container
+CONTAINER_NAME_FIELD = "containerName"
+CONTAINER_ID_FIELD = "containerID"
+CONTAINER_AGE = "age"
+CONTAINER_IMAGE_NAME = "image-name"
+# Backend Container Fields
+BE_CONTAINER_IMAGE = "image"
+BE_CONTAINER_IMAGE_ID = "image_id"
+BE_CONTAINER_NAME = "container_name"
+BE_CONTAINER_ID = "container_id"
+
+# Agents
+AGENT_HEALTH_CRIT = "Critical"
+AGENT_HEALTH_DEL = "Deleted"
+AGENT_HEALTH_ERR = "Error"
+AGENT_HEALTH_NORM = "Normal"
+AGENT_HEALTH_OFFLINE = "Offline"
+AGENT_HEALTH_RESTARTED = "Restarted"
+AGENT_HEALTH_RESTARTING = "Restarting"
+AGENT_HEALTH_STARTED = "Started"
+AGENT_HEALTH_STARTING = "Starting"
+AGENT_HEALTH_WARN = "Warning"
+HEALTH_PRIORITY = {
+    AGENT_HEALTH_CRIT: 20,
+    AGENT_HEALTH_ERR: 30,
+    AGENT_HEALTH_WARN: 40,
+    AGENT_HEALTH_NORM: 50,
+    AGENT_HEALTH_OFFLINE: 10,
+    AGENT_HEALTH_RESTARTED: 70,
+    AGENT_HEALTH_RESTARTING: 60,
+    AGENT_HEALTH_STARTED: 90,
+    AGENT_HEALTH_STARTING: 80,
+    AGENT_HEALTH_DEL: 100,
+}
+AGENT_STATUS = "status"
+AGENT_HOSTNAME = "hostname"
+AGENT_ID = "id"
+AGENT_BAT_STATUSES = "bat_statuses"
+
+# Spydertraces
+BE_TRIGGER_NAME = "trigger_short_name"
+BE_SCORE = "score"
+BE_SUPPRESSED = "suppressed"
+BE_ROOT_PROC_NAME = "root_proc_name"
+BE_UNIQUE_FLAG_COUNT = "unique_flag_count"
+BE_OBJECT_COUNT = "object_count"
+BE_PROCESSES = "processes"
+BE_DEPTH = "depth"
+BE_SYSTEMS = "machines"
+BE_CONNECTIONS = "connections"
+
 # Network
 CIDR_FIELD = "cidr"
 EGRESS_FIELD = "egress"
@@ -442,6 +866,9 @@ PROCESSES_FIELD = "processes"
 PROTO_FIELD = "protocol"
 TO_FIELD = "to"
 
+# Flags
+FLAG_CLASS = "class"
+
 # Output
 OUTPUT_YAML = "yaml"
 OUTPUT_JSON = "json"
@@ -599,15 +1026,19 @@ def load_file(path: Path) -> Dict:
 
 class CustomGroup(click.Group):
     SECTION_BASIC = "Basic Commands"
+    SECTION_ALERT_MGMT = "Alert Management"
     SECTION_OTHER = "Other Commands"
-    command_sections = [SECTION_BASIC, SECTION_OTHER]
+    command_sections = [SECTION_BASIC, SECTION_ALERT_MGMT, SECTION_OTHER]
     cmd_to_section_map = {
         "apply": SECTION_BASIC,
         "create": SECTION_BASIC,
+        "close": SECTION_ALERT_MGMT,
         "delete": SECTION_BASIC,
         "diff": SECTION_BASIC,
         "get": SECTION_BASIC,
         "merge": SECTION_BASIC,
+        "snooze": SECTION_ALERT_MGMT,
+        "suppress": SECTION_ALERT_MGMT,
         "validate": SECTION_BASIC,
     }
 
@@ -979,6 +1410,27 @@ class MutuallyExclusiveEatAll(MutuallyExclusiveOption, OptionEatAll):
 
 
 def try_log(*args, **kwargs):
+    global LOG_VAR
+    try:
+        if kwargs.pop(WARNING_MSG, False):
+            msg = f"{WARNING_COLOR}{' '.join(args)}{COLOR_END}"
+            if USE_LOG_VARS:
+                LOG_VAR.append(msg)
+            print(msg, **kwargs, file=sys.stderr)
+        else:
+            msg = " ".join(args)
+            if USE_LOG_VARS:
+                LOG_VAR.append(msg)
+            print(*args, **kwargs, file=sys.stderr)
+        sys.stderr.flush()
+    except BrokenPipeError:
+        devnull = os.open(os.devnull, os.O_WRONLY)
+        os.dup2(devnull, sys.stderr.fileno())
+        err_exit("Broken Pipe")
+
+
+def api_log(*args, **kwargs):
+    global LOG_VAR
     try:
         if kwargs.pop(WARNING_MSG, False):
             msg = f"{WARNING_COLOR}{' '.join(args)}{COLOR_END}"
@@ -989,10 +1441,10 @@ def try_log(*args, **kwargs):
     except BrokenPipeError:
         devnull = os.open(os.devnull, os.O_WRONLY)
         os.dup2(devnull, sys.stderr.fileno())
-        sys.exit(1)
+        err_exit("Broken Pipe")
 
 
-def time_inp(time_str: str, cap_one_day=True) -> int:
+def time_inp(time_str: str, cap_one_day=False) -> int:
     past_seconds = 0
     epoch_time = None
     try:
@@ -1020,14 +1472,12 @@ def time_inp(time_str: str, cap_one_day=True) -> int:
     if epoch_time is not None:
         if epoch_time > now:
             raise ValueError("time must be in the past")
-        # TODO: Make API calls robust to times older than one day
         if epoch_time < one_day_ago and cap_one_day:
             epoch_time = one_day_ago
         return epoch_time
     else:
         if past_seconds < 0:
             raise ValueError("time must be in the past")
-        # TODO: Make API calls robust to times older than one day
         if past_seconds > 86400 and cap_one_day:
             past_seconds = 86400
         return int(now - past_seconds)
@@ -1204,7 +1654,9 @@ def make_uuid():
     return b64url(uuid4().bytes).decode("ascii").strip("=")
 
 
-def err_exit(message: str):
+def err_exit(message: str, exception: Exception = None):
+    if API_CALL:
+        raise Exception(f"{WARNING_COLOR}Error: {message}{COLOR_END}")
     sys.exit(f"{WARNING_COLOR}Error: {message}{COLOR_END}")
 
 
@@ -1235,7 +1687,70 @@ class UniqueKeyLoader(yaml.SafeLoader):
         return super().construct_mapping(node, deep)
 
 
-def load_resource_file(file: Union[str, IO]):
+def load_resource_file(file: Union[str, IO], validate_cmd: bool = False):
+    try:
+        name, resrc_data = __load_yaml_file(file)
+    except ValueError as e:
+        if validate_cmd:
+            try_log(" ".join(e.args))
+            sys.exit(0)
+        err_exit(" ".join(e.args))
+    except Exception as e:
+        if file.name.endswith(".yaml"):
+            err_exit("Error decoding yaml" + str(e.args))
+        try:
+            name, resrc_data = __load_json_file(file)
+        except json.JSONDecodeError as e:
+            if validate_cmd:
+                try_log("Error decoding json" + " ".join(e.args))
+                sys.exit(0)
+            err_exit("Error decoding json" + " ".join(e.args))
+        except ValueError as e:
+            if validate_cmd:
+                try_log(" ".join(e.args))
+                sys.exit(0)
+            err_exit(" ".join(e.args))
+        except Exception:
+            err_exit("Unable to load resource file.")
+    __validate_data_structure_on_load(resrc_data, validate_cmd)
+    if isinstance(resrc_data, dict):
+        __validate_resource_on_load(resrc_data, name, validate_cmd)
+    else:
+        for i, data in enumerate(resrc_data):
+            __validate_resource_on_load(data, name, validate_cmd, index=i)
+    if isinstance(file, io.TextIOWrapper):
+        file.seek(0, 0)
+    return resrc_data
+
+
+def __validate_data_structure_on_load(resrc_data: Any, validate_cmd=False):
+    if not isinstance(resrc_data, dict) and not isinstance(resrc_data, list):
+        if validate_cmd:
+            try_log(
+                "Resource file does not contain a dictionary or list of"
+                " dictionaries."
+            )
+            sys.exit(0)
+        err_exit(
+            "Resource file does not contain a dictionary or list of"
+            " dictionaries."
+        )
+
+
+def __validate_resource_on_load(
+    resrc_data: Dict, name, validate_cmd=False, index=None
+):
+    msg_suffix = "" if index is None else f" at index {index}"
+    from spyctl.schemas_v2 import valid_object
+
+    if not valid_object(resrc_data, verbose=True):
+        if validate_cmd:
+            try_log(f"Invalid object in {name!r}{msg_suffix}. See error logs.")
+            sys.exit(0)
+        err_exit(f"Invalid object in {name!r}{msg_suffix}. See error logs.")
+
+
+def __load_yaml_file(file: Union[str, IO]) -> Tuple[str, Any]:
     try:
         if isinstance(file, io.TextIOWrapper):
             name = file.name
@@ -1244,37 +1759,27 @@ def load_resource_file(file: Union[str, IO]):
             name = file
             with open(file) as f:
                 resrc_data = yaml.load(f, UniqueKeyLoader)
-    except ValueError as e:
+    except IOError as e:
         err_exit(" ".join(e.args))
-    except Exception:
-        try:
-            if isinstance(file, io.TextIOWrapper):
-                name = file.name
-                resrc_data = json.load(
-                    file, object_pairs_hook=dict_raise_on_duplicates
-                )
-            else:
-                name = file
-                with open(file) as f:
-                    resrc_data = json.load(
-                        f, object_pairs_hook=dict_raise_on_duplicates
-                    )
-        except ValueError as e:
-            err_exit(" ".join(e.args))
-        except Exception:
-            err_exit("Unable to load resource file.")
-    if not isinstance(resrc_data, dict):
-        err_exit("Resource file does not contain a dictionary.")
-    resrc_kind = resrc_data.get(KIND_FIELD)
-    if not resrc_kind:
-        err_exit(f"Missing or invalid {KIND_FIELD} field.")
-    from spyctl.schemas import valid_object
+    return name, resrc_data
 
-    if not valid_object(resrc_data, verbose=True):
-        sys.exit(f"{resrc_kind} invalid in {name!r}. See error logs.")
-    if isinstance(file, io.TextIOWrapper):
-        file.seek(0, 0)
-    return resrc_data
+
+def __load_json_file(file: Union[str, IO]) -> Tuple[str, Any]:
+    try:
+        if isinstance(file, io.TextIOWrapper):
+            name = file.name
+            resrc_data = json.load(
+                file, object_pairs_hook=dict_raise_on_duplicates
+            )
+        else:
+            name = file
+            with open(file) as f:
+                resrc_data = json.load(
+                    f, object_pairs_hook=dict_raise_on_duplicates
+                )
+    except IOError as e:
+        err_exit(" ".join(e.args))
+    return name, resrc_data
 
 
 def dictionary_mod(fn) -> Dict:
@@ -1292,11 +1797,24 @@ def dictionary_mod(fn) -> Dict:
 
 
 def to_timestamp(zulu_str):
-    return zulu.Zulu.parse(zulu_str).timestamp()
+    try:
+        return zulu.Zulu.parse(zulu_str).timestamp()
+    except Exception:
+        return zulu_str
 
 
 def epoch_to_zulu(epoch):
-    return zulu.Zulu.fromtimestamp(epoch).format("YYYY-MM-ddTHH:mm:ss") + "Z"
+    try:
+        return (
+            zulu.Zulu.fromtimestamp(epoch).format("YYYY-MM-ddTHH:mm:ss") + "Z"
+        )
+    except Exception:
+        return epoch
+
+
+def truncate_hour_epoch(input_epoch: float) -> float:
+    rv = input_epoch - (input_epoch % 3600)
+    return rv
 
 
 def get_metadata_name(resource: Dict) -> Optional[str]:
@@ -1372,3 +1890,78 @@ def unique_fn(fn: str, output_format) -> Optional[str]:
         try_log(f"Unable to build unique filename for {fn}", is_warning=True)
         return
     return str(new_fn)
+
+
+def make_checksum(dictionary: Dict) -> str:
+    dict_str = json.dumps(dictionary, sort_keys=True)
+    hash = md5(dict_str.encode("utf-8"))
+    return hash.hexdigest()
+
+
+def simple_glob_to_regex(input_str: str):
+    rv = input_str.replace(".", "\\.")
+    rv = rv.replace("^", "\\^")
+    rv = rv.replace("$", "\\$")
+    rv = rv.replace("*", ".*")
+    rv = rv.replace("?", ".")
+    rv = f"^{rv}$"
+    return rv
+
+
+def set_api_call():
+    global API_CALL, USE_LOG_VARS
+    API_CALL = True
+    USE_LOG_VARS = True
+    disable_colorization()
+
+
+def set_debug():
+    global DEBUG
+    DEBUG = True
+
+
+def load_file_for_api_test(file: IO):
+    try:
+        _, resrc_data = __load_yaml_file(file)
+    except ValueError as e:
+        err_exit(" ".join(e.args))
+    except Exception:
+        try:
+            _, resrc_data = __load_json_file(file)
+        except json.JSONDecodeError as e:
+            err_exit("Error decoding json" + " ".join(e.args))
+        except ValueError as e:
+            err_exit(" ".join(e.args))
+        except Exception:
+            err_exit("Unable to load resource file.")
+    return resrc_data
+
+
+def is_private_dns(hostname: str) -> bool:
+    if hostname.endswith(".local"):
+        return True
+    return False
+
+
+def is_public_dns(hostname: str) -> bool:
+    if not is_private_dns(hostname):
+        return True
+    return False
+
+
+def is_redirected() -> bool:
+    return os.fstat(0) != os.fstat(1)
+
+
+def calc_age(time_float: float):
+    creation_timestamp = zulu.parse(time_float)
+    age_delta = zulu.now() - creation_timestamp
+    if age_delta.days > 0:
+        age = f"{age_delta.days}d"
+        return age
+    elif age_delta.seconds >= 3600:
+        age = f"{age_delta.seconds // 3600}h"
+        return age
+    else:
+        age = f"{age_delta.seconds//60}m"
+        return age
